// refer to siomn's fxDocumentation
#ifndef SYSCALL
#define SYSCALL

int RTC_GetTicks(void);  // 1/128s counter
void RTC_SetDateTime(unsigned char* timestr);

int Timer_Install(int InternalTimerID, void *handler, int elapse);
int Timer_Deinstall(int InternalTimerID);
int Timer_Start(int InternalTimerID);
int Timer_Stop(int InternalTimerID);

void PutKey(unsigned int keycode, int mode);
void Keyboard_ClrBuffer(void);
int Keyboard_PutMatrixCode(short *matrixCode);
int Keyboard_PutKeyCode(int X, int Y, int KeyCode);
int Keyboard_GetKeyWait(int* column, int* row, int type_of_waiting, int timeout_period, int menu, unsigned short* keycode);
//void EnableGetkeyToMainFunctionReturn(void);	//Enable [Menu] key to exit to Main Menu
void EnableGetkeyToMainFunctionReturn(void); 
//void DisableGetkeyToMainFunctionReturn(void);	//Disable [Menu] key from exiting to Main Menu
void DisableGetkeyToMainFunctionReturn(void); 

typedef struct {
    int flashstyle;     // -1 if not flashing, else flashmode
    int column;         // starts at 1
    int row;            // starts at 1
    int graphic_mode;
} TCursorSettings;
int Cursor_SetPosition(char x, char y); 
int Cursor_SetFlashStyle(short int flashstyle);
void Cursor_SetFlashMode(long flashmode);
unsigned int Cursor_GetSettings(unsigned int settingsarray[]);
int Cursor_GetSettings_Struct(TCursorSettings *CursorSettings);
int Cursor_GetFlashStyle(void);
void Cursor_SetFlashOn(char flash_style);
void Cursor_SetFlashOff(void); 

int OpcodeToStr( unsigned short opcode, unsigned char *string );

int MB_IsLead(char character);
int MB_ElementCount(char* str);
int MB_ByteCount(char* str);
char* MB_strcat(char* dest, char* src);
char* MB_strncat(char* dest, char* src, int bytes);
char* MB_strcpy(char* dest, char* src);
char* MB_GetSecondElemPtr(char* str);
short int MB_GetElement(char* str);
char* MB_CopyToHeap(char* src);

void GetFKeyIconPointer(int FKeyNo, unsigned char* pBitmap);
void DisplayFKeyIcon(int FKeyPos, unsigned char* pBitmap);
void GUI_ProgressBar(int current, int max);

void System_GetOSVersion(unsigned char version[10]);
void PowerOff(int logo);    //logo = 0 : Turn calc off, logo = 1 : Turn calc off with logo

//------------------------------------------------------- MCS
typedef struct {
    unsigned char name[8];
    void* addr;
    unsigned short count;
    char flags[2];
} TMainMemoryDirectoryEntry;

typedef struct {
    unsigned char name[8];
    int offset;
    int length;
    char flags[4];
} TDirectoryItem;

int MCS_CreateDirectory( unsigned char*dir, char*dirno );
int MCS_WriteItem( unsigned char*dir, unsigned char*item, short itemtype, int data_length, int buffer );
int MCS_DeleteDirectory( unsigned char*dir );
int MCS_DeleteItem( unsigned char*dir, unsigned char*item );
int MCS_GetState( int*maxspace, int*currentload, int*remainingspace );
int MCS_GetSystemDirectoryInfo( unsigned char*dir, unsigned int*pdir, char*dirno );
int MCS_RenameItem( unsigned char*srcdir, unsigned char*srcitem, unsigned char*tgtdir, int tgtitem );
int MCS_OverwriteData( unsigned char*dir, unsigned char*item, int write_offset, int bytes_to_write, void*buffer );
int MCS_GetItemData( unsigned char*dir, unsigned char*item, int offset, int bytes_to_read, void*buffer );
int MCS_RenameDirectory( unsigned char*oldname, unsigned char*newname );
int MCS_SearchDirectory( unsigned char*dir, TMainMemoryDirectoryEntry*pdir, char*dirno );
//int MCS_SearchDirectoryItem(unsigned char* dir, unsigned char* item, char* flags_0, TDirectoryItem* item_ptr, int* data_ptr, int* data_length);
int MCS_GetFirstDataPointerByDirno( char*dirno, void*pdata );
int MCS_GetDirectoryEntryByNumber( char dirno, TMainMemoryDirectoryEntry*pdir );
int MCS_SearchItem( unsigned char*item, TMainMemoryDirectoryEntry*pdir, unsigned short*itemno );
int MCS_str8cpy( unsigned char*source, unsigned char*target, int mode );
void MCS_GetDirectoryEntryAddress( void*directory_entry_address );
void MCS_GetCurrentBottomAddress( void*current_bottom_address );
int MCS_GetCapa( int*current_bottom );
int MCS_GetMainMemoryStart( void );
int Bfile_RenameEntry( const unsigned short *oldname, const unsigned short *newname );
int DD_SetContrast(int value);
//------------------------------------------------------- Bdisp
unsigned char* GetVRAMAddress(void);

typedef struct {
    unsigned int x1;
    unsigned int y1;
    unsigned int x2;
    unsigned int y2;
    unsigned char f[4];
    unsigned int on_bits;
    unsigned int off_bits;
} TShape;

void Bdisp_ShapeToVRAM(TShape *p);

//------------------------------------------------------- Call

int Calljmp(int r4, int r5, int r6, int r7, int adrs);
int SysCalljmp(int r4, int r5, int r6, int r7, int no);
int SysCalljmp12(int r1, int r2, int r3, int r4, int r5, int r6, int r7, int r8, int r9, int r10, int r11, int r12, int No);

//------------------------------------------------------- Serial
int Serial_Open2( unsigned short parameters );	//0x02AB
int Serial_Open_57600( void );	//0x0285
void*Serial_ResetAndDisable( void );	//0x0409
void*Serial_GetInterruptHandler( int type );	//0x040A
int Serial_SetInterruptHandler( int type, void*handler );	//0x040B
int Serial_ReadOneByte( unsigned char*result );	//0x040C
int Serial_ReadNBytes( unsigned char*result, int max_size, short*actually_transferred );	//0x040D
int Serial_BufferedTransmitOneByte( unsigned char byte_to_transmit );	//0x040E
int Serial_BufferedTransmitNBytes( unsigned char*bytes_to_transmit, int requested_count );	//0x040F
int Serial_DirectTransmitOneByte( unsigned char byte_to_transmit );	//0x0410
int Serial_GetReceivedBytesAvailable( void );	//0x0411
int Serial_GetFreeTransmitSpace( void );	//0x0412
int Serial_ClearReceiveBuffer( void );	//0x0413
int Serial_ClearTransmitBuffer( void );	//0x0414
int Serial_Open( unsigned char *mode );	//0x0418
int Serial_Close( int mode );	//0x0419
void*Serial_CallReceiveIntErrorResetHandler( void );	//0x041B
void*Serial_CallReceiveIntHandler( void );	//0x041C
void*Serial_CallTransmitIntErrorResetHandler( void );	//0x041D
void*Serial_CallTransmitIntHandler( void );	//0x041E
int Serial_SpyNthByte( int byteno_to_spy, unsigned char*result );	//0x0422
void Serial_GetStatus( unsigned int*serial_status );	//0x0423 
int Serial_IsOpen( void );	// 0x0425


//Dmarre l'application numro 'appno' (commence  0)
//P1, P2 et P4 doivent toujours etre  0;
//Ne fonctionne pas sur l'mulateur
int StartAnyApp(int P1, int P2, int appno, int P4);

// Adds keycode to the key buffer as if the user pressed the physical key
// mode is unknown. Works with 0
void KBD_PutKey(unsigned int keycode, int mode);

//Scan les addin de la mmoire de stockage
//Exemple, si votre addin copie un .G1A dans la mmoire de stockage (depuis la carte SD par exemple)
//il faut appeler Register_addin() avant de pouvoir lancer ce .G1A avec StartAnyApp()
void Register_Addin(void);

//Retourne l'adresse de la VRAM, quelque soit l'OS (fonctionne aussi sur l'mulateur)
char* VRam_Base(void);


/***********************************************************************/
/** Les 3 syscalls suivants servent a acceder  la mmoire principale **/
/** Je n'ai pas beaucoup d'informations,                              **/
/** je ne vous conseil pas de les utiliser avant d'en savoir plus.    **/
/***********************************************************************/

//Cherche un lment dans un dossier de la mmoire principale.
//Exemple: "system" pour les programmes BASIC, "@REV2" pour les pictures
int MCS_SearchDirectoryItem(unsigned char* dir, unsigned char* item, char* flags_0, void* item_ptr, int* data_ptr, int* data_length);

//Cre un fichier dans un dossier de la mmoire principale et crit dedans.
//dirtype=1 pour le dossier "system", c'est le vrai nom du dossier des programmes BASIC
int MCS_PutInternalItem(char dirtype, unsigned char* item, int data_len, void* buffer);

//Supprime un lment d'un dossier de la mmoire principale
int MCS_DeleteInternalItem(char dirtype, unsigned char* item);


/******************************************************************/
/** TOUS les syscalls suivants sont tirs de fxreverse-doc-1.pdf **/
/******************************************************************/

//Description: Moves the cursor to the given position, where column is in the range[0, 20] and row is
//in the range[0, 7].The oXcial SDK function locate() is a wrapper for this function.
//Returns : 1 on success, 0 if an argument is out of range
int Cursor_SetPosition(char column, char row);

//Dfini le style du curser (0=normal, 1=SHIFT, 2=ALPHA)
//Retourne 1 si a a march, 0 si l'argument n'est pas bon
int Cursor_SetFlashStyle(short int flashstyle);

//Dfini le mode de clignottement du curseur. Si flashmode!=0, le curseur clignotte, si flashmode==0, le curseur ne clignotte plus
void Cursor_SetFlashMode(long flashmode);

//Permet de rcuprer des infos sur le style du curseur
//settingsarray contient dans cet order : colonne, ligne, flash style, fhash mode (il faut donc un tableau de 4 int)
//Retourne : on n'en sait rien
unsigned int Cursor_GetSettings(unsigned int settingsarray[]);

//Equivalent de Cursor_SetFlashMode(0);
void Cursor_SetFlashOff(void);


/*************************************************************************/
/** Les syscalls suivants servent a appeler les applications du systme **/
/*************************************************************************/

//Les arguments sont les mmes que pour AddIn_main, si runMode vaut 0, c'est qu'on appelle l'application depuis une e-activity
//Dans ce cas, stripNum donne le numro du strip de l'e-activity
//Pour excuter normalement une application, mettez runMode  1
void App_CONICS(int runMode, unsigned short stripNum);
void App_DYNA(int runMode, unsigned short stripNum);
void App_EACT(int runMode, unsigned short stripNum);
void App_EQUA(int runMode, unsigned short stripNum);
void App_PRGM(int runMode, unsigned short stripNum);
void App_FINANCE(int runMode, unsigned short stripNum);
void App_GRAPH(int runMode, unsigned short stripNum);
void App_LINK(int runMode, unsigned short stripNum);
void App_MEMORY(int runMode, unsigned short stripNum);
void App_RECUR(int runMode, unsigned short stripNum);
void App_RUNMAT(int runMode, unsigned short stripNum);
void App_STAT(int runMode, unsigned short stripNum);
void App_SYSTEM(int runMode, unsigned short stripNum);


/**********************************************************/
/** Les syscall suivants servent  l'acces au port srie **/
/** Le buffer de rception fait 1ko,                     **/
/** et le buffer d'envoi fait 256 octets.                **/
/**********************************************************/

//Lit un caractre du buffer de rception et le copie a l'adresse pointe par 'dest'
//Retourne 0 en cas de succes, 1 si le buffer est vide, 3 si la connexion n'est pas tablie
int Serial_ReadByte(unsigned char* dest);

//Lit 'max' octets du buffer de rception et les crit dans 'dest'
//'size' donne le nombre d'octets lu
//Retourne 0 en cas de succes, 1 si le buffer est vide, 3 si la connexion n'est pas tablie
int Serial_ReadBytes(unsigned char* dest, int max, short* size);

//Ecrit 'byte' dans le buffer d'envoi
//Retourne 0
int Serial_WriteByte(unsigned char byte);

//Ecrit 'size' octets dans le buffer d'envoi depuis l'adresse 'scr'
//Retourne 0 en cas de succes, 2 si le buffer est trop plein, 3 si la connexion n'est pas tablie
int Serial_WriteBytes(unsigned char* src, int size);

//FIFO = first in first out
int Serial_WriteByteFIFO(unsigned char byte);

//Retourne la taille des donnes du buffer de rception
int Serial_GetRxBufferSize(void);

//Retourne l'espace disponible dans le buffer d'envoi
int Serial_GetTxBufferFreeCapacity(void);

//Vide le buffer de rception
//Retourne 0 en cas de succes, 3 si la connexion n'est pas tablie
int Serial_ClearReceiveBuffer(void);

//Vide le buffer d'envoi
//Retourne 0
int Serial_ClearTransmitBuffer(void);

//Ouvre et prpare l'interface de communication
//Pour plus de dtails, consulter fxreverse-doc-1.pdf ci joint
int Serial_Open(unsigned char* conf);

//Ferme l'interface de communication et vide les buffers d'envoi et de rception
//Si 'mode'==1, la communication est coupe sans regarder s'il reste des donnes a transmettre
//Si 'mode'!=1, la fonction ne ferme pas l'interface de communication s'il reste des donnes a transmettre
//et retourne 5
//Retourne 0 en cas de succes (communication termine) et 5 s'il reste des donnes a transmettre
int Serial_Close(int mode);

//Copie l'octet numro 'index' du buffer de rception vers 'dest' si 'index' ne dpasse pas les donnes du buffer
//Retourne 0 en cas de succes, 1 si 'index' dpasse les donnes du buffer, 3 si la communication n'est pas tablie
int Serial_Peek(int index, unsigned char* dest);

//Rcupre le statut de la connexion
//Retourne 1 si la connexion est tablie, 3 sinon
int Serial_IsOpen(void);

/*********************************************************************/
/** Les syscalls suivants servent a l'acces aux variables Setup     **/
/** Il est prcis dans la doc que ces syscall sont valides         **/
/** pour l'OS 1.2, mais peut etre pas pour les versions suprieures **/
/*********************************************************************/

//Lit la variable Setup numro 'index'
//Retourne la valeur de la variables Setup numro 'index'
char Setup_GetEntry(unsigned int index);

//Ecrit la valeur 'value' dans la variables Setup numro 'index'
//Retourne l'adresse du tableau des variables Setup
char* Setup_SetEntry(unsigned int index, char value);

//Retourne l'adresse de la variable Setup numro 'index'
char* Setup_GetEntryPtr(unsigned int index);

/*************************************************************************/
/** Les syscalls suivants servent a acceder aux variables               **/
/** utilises par les programmes BASIC, A-Z, r, theta et Ans.           **/
/** Les variables A-Z sont reprsentes par leur code ASCII (0x41-0x5A) **/
/** r, thete et Ans sont reprsentes par les codes 0xCE, 0xCD et 0xC0  **/
/*************************************************************************/

//Lit la variable 'variable' et copie sa valeur dans 'dest'
//'dest' doit etre un tableau de 24 octets
//Retourne le pointeur envoy, 'dest'
char* Alpha_GetData(char variable, char* dest);

//Ecrit le contenu de 'scr' dans la variable 'variable'
//'scr doit etre un tableau de 24 octets
//Si 'variable' est valide, retourne 0, sinon retourne 'variable'
char Alpha_SetData(char variable, char* src);

//Initialise toutes les variables Alpha, except Ans
void Alpha_ClearAll(void);




#endif // !SYSCALL